/*
 * Copyright (c) 2016, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.wso2.carbon.device.mgt.mobile.impl.ios.profile;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.device.mgt.ios.payload.beans.DeviceProfile;
import org.wso2.carbon.device.mgt.ios.payload.exception.PListException;
import org.wso2.carbon.device.mgt.mobile.impl.ios.dao.AbstractMobileDeviceManagementDAOFactory;
import org.wso2.carbon.device.mgt.mobile.impl.ios.dao.IOSDAOFactory;
import org.wso2.carbon.device.mgt.mobile.impl.ios.dao.MobileDeviceManagementDAOException;
import org.wso2.carbon.device.mgt.mobile.impl.ios.dao.impl.ProfileDAO;

public class IOSProfileServiceImpl implements IOSProfileService {

    private static final Log log = LogFactory.getLog(IOSProfileServiceImpl.class);
    private static AbstractMobileDeviceManagementDAOFactory mobileDeviceManagementDAOFactory;

    public IOSProfileServiceImpl() {
        mobileDeviceManagementDAOFactory = new IOSDAOFactory();
    }

    @Override
    public void saveProfile(DeviceProfile deviceProfile) throws PListException {
        try {
            IOSDAOFactory.beginTransaction();
            ProfileDAO profileDAO = new ProfileDAO();
            profileDAO.addProfile(deviceProfile);
            IOSDAOFactory.commitTransaction();
        } catch (MobileDeviceManagementDAOException e) {
            try {
                IOSDAOFactory.rollbackTransaction();
            } catch (MobileDeviceManagementDAOException e1) {
                log.warn("Error occurred while roll-backing the transaction", e);
            }
            throw new PListException("Error occurred while adding the profile", e);
        }
    }

    @Override
    public DeviceProfile getProfile(String deviceId, String url) throws PListException{
        DeviceProfile deviceProfile;
        try {
            IOSDAOFactory.beginTransaction();
            ProfileDAO profileDAO = new ProfileDAO();
            deviceProfile = profileDAO.getProfile(deviceId, url);
            IOSDAOFactory.commitTransaction();
        } catch (MobileDeviceManagementDAOException e) {
            try {
                IOSDAOFactory.rollbackTransaction();
            } catch (MobileDeviceManagementDAOException e1) {
                log.warn("Error occurred while roll-backing the transaction", e);
            }
            throw new PListException("Error occurred while adding the profile", e);
        }
        return deviceProfile;
    }
}
